'use strict';

import GObject from 'gi://GObject?version=2.0';
import St from 'gi://St';  // Version belirtmiyoruz
import Gio from 'gi://Gio?version=2.0';
import GLib from 'gi://GLib?version=2.0';
import Soup from 'gi://Soup?version=3.0';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
import {Extension} from 'resource:///org/gnome/shell/extensions/extension.js';

const INDICATOR_ICON = 'currency-tracker-symbolic';

const Indicator = GObject.registerClass(
class Indicator extends PanelMenu.Button {
    _init() {
        super._init(0.0, 'Currency Tracker');
        
        this._panelBox = new St.BoxLayout({
            style_class: 'panel-status-menu-box'
        });
        
        this._icon = new St.Icon({
            icon_name: 'office-database-symbolic',
            style_class: 'system-status-icon currency-tracker-icon'
        });
        
        this._label = new St.Label({
            text: 'Loading...',
            y_align: Clutter.ActorAlign.CENTER,
            style_class: 'currency-tracker-label'
        });
        
        this._panelBox.add_child(this._icon);
        this._panelBox.add_child(this._label);
        this.add_child(this._panelBox);
        
        this._buildMenu();
        this._refresh();
    }

    _buildMenu() {
        const refreshItem = new PopupMenu.PopupMenuItem('Refresh');
        refreshItem.connect('activate', () => {
            this._refresh();
        });
        this.menu.addMenuItem(refreshItem);
    }

    async _refresh() {
        try {
            const response = await this._fetchData('USD-TRY');
            if (response) {
                const text = new TextDecoder().decode(response);
                const data = JSON.parse(text);
                if (data && data.USDTRY) {
                    this._label.set_text(`USD/TRY: ${data.USDTRY.bid}`);
                }
            }
        } catch (error) {
            this._label.set_text('Error');
            console.error(error);
        }
    }

    async _fetchData(pair) {
        try {
            const url = `https://economia.awesomeapi.com.br/json/last/${pair}`;
            const session = new Soup.Session();
            const message = Soup.Message.new('GET', url);
            
            if (!message) {
                throw new Error('Failed to create HTTP message');
            }

            const bytes = await session.send_and_read_async(
                message,
                GLib.PRIORITY_DEFAULT,
                null
            );

            if (!bytes) {
                throw new Error('No response data');
            }

            return bytes.get_data();
        } catch (error) {
            console.error('Fetch error:', error);
            return null;
        } finally {
            if (session) {
                session.abort();
            }
        }
    }
});

export default class CurrencyTracker extends Extension {
    enable() {
        console.log('Currency Tracker: enabling...');
        this._indicator = new Indicator();
        Main.panel.addToStatusArea('currency-tracker', this._indicator);
    }

    disable() {
        console.log('Currency Tracker: disabling...');
        if (this._indicator) {
            this._indicator.destroy();
            this._indicator = null;
        }
    }
}